﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Visualization;
using HeuristicLab.Visualization.Primitives;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views {
  public class SymbolicExpressionTreeTile : Group {
    public double PreferredNodeWidth { get; set; }
    public double PreferredNodeHeight { get; set; }
    private ISymbolicExpressionTree symbolicExpressionTree;
    public ISymbolicExpressionTree SymbolicExpressionTree {
      get { return symbolicExpressionTree; }
      set {
        symbolicExpressionTree = value;
        GeneratePrimitives(PreferredNodeWidth, PreferredNodeHeight);
      }
    }

    public ReingoldTilfordLayoutEngine<ISymbolicExpressionTreeNode> LayoutEngine { get; set; }
    public SymbolicExpressionTreeLayoutAdapter LayoutAdapter { get; set; }
    public SymbolicExpressionTreeTile(IChart chart) : base(chart) { }
    public SymbolicExpressionTreeTile(IChart chart, ISymbolicExpressionTree tree)
      : base(chart) {
      SymbolicExpressionTree = tree;
      PreferredNodeWidth = 80;
      PreferredNodeHeight = 40;
    }
    private void GeneratePrimitives(double preferredNodeWidth, double preferredNodeHeight) {
      Clear();
      var layoutNodes = LayoutAdapter.Convert(SymbolicExpressionTree).ToList();
      layoutNodes.RemoveAt(0);
      LayoutEngine.Root = layoutNodes[0];
      LayoutEngine.CalculateLayout();

      var primitivesMap = new Dictionary<LayoutNode<ISymbolicExpressionTreeNode>, IPrimitive>(); // both Ellipse and Rectangle are derived from the RectangularPrimitiveBase
      var font = new Font(FontFamily.GenericSansSerif, 10, GraphicsUnit.Pixel);

      foreach (var node in layoutNodes) {
        var lowerLeft = new PointD(node.X, node.Y);
        var upperRight = new PointD(node.X + preferredNodeWidth, node.Y + preferredNodeHeight);

        RectangularPrimitiveBase rectangularPrimitive;
        if (node.IsLeaf) {
          rectangularPrimitive = new LabeledRectangle(Chart, lowerLeft, upperRight) { Font = font, Text = node.Content.ToString() };
        } else {
          rectangularPrimitive = new LabeledEllipse(Chart, lowerLeft, upperRight) { Font = font, Text = node.Content.ToString() };
        }

        this.Add(rectangularPrimitive);
        primitivesMap.Add(node, rectangularPrimitive);
      }

      foreach (var node in layoutNodes.Where(n => n.Children != null)) {
        foreach (var child in node.Children) {
          var start = new PointD(node.X + preferredNodeWidth / 2, node.Y + preferredNodeHeight);
          var end = new PointD(child.X + preferredNodeWidth / 2, child.Y);
          var line = new Line(this.Chart, start, end);
          this.Add(line);
        }
      }
    }
  }
}
